﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using GenericParsing;

/*Add this line to use ExcelWriter*/
using SoftArtisans.OfficeWriter.ExcelWriter;

namespace ExtendedSalesSummary
{
    public partial class Part1_CSV : System.Web.UI.Page
    {
        /*Declare the global variables that will be used throughout the code.
        * This includes the ExcelApplication object, the ExcelTemplate object,
        * and the Workbook object.*/
        private ExcelApplication xla;
        private ExcelTemplate xlt;
        private Workbook wb;

        protected void Page_Load(object sender, EventArgs e)
        {
        }


        protected void RunReport_Click(object sender, EventArgs e)
        //This method is run when the "Run Report" Button is clicked
        {
            try
            {
                //Generate the report
                GenerateReport();
            }
            catch (Exception ex)
            {
                ErrorLabel.Visible = true;
                if (ex.Message.Contains("General license key exception"))
                {
                    ErrorLabel.Text = "OfficeWriter does not appear to be" +
                       " installed on this machine. If you have not " +
                       "purchased OfficeWriter, you can download the " +
                       "evaluation version here: " +
                       "http://www.officewriter.com/officewriter-free-evaluation.aspx";
                }
                else
                {
                    ErrorLabel.Text = ex.Message;
                }

            }
        }

        protected void GenerateReport()
        {
            //Format the template to deal with many different countries
            GenerateTemplate();

            //Populate the template by binding the data
            PopulateTemplate();

            //Process and stream the final report
            xlt.Process();
            xlt.Save(Page.Response, "output.xlsx", false);
        }


        protected void PopulateTemplate()
        {
            //Create a new ExcelTemplate object and open the existing 
            //ExcelApplication.
            xlt = new ExcelTemplate();
            xlt.Open(xla, wb);

            //Iterate through the worksheets using a for loop. 
            //For each worksheet in the workbook, bind the country data
            //using the BindCountryData function. This function requires the name of
            //the worksheet as a string parameter.
            for (int i = 1; i < wb.Worksheets.Count; i++)
            {
                BindCountryData(wb.Worksheets[i].Name);
            }
        }


        public void GenerateTemplate()
        {
            //Create a new ExcelApplication and open the existing template 
            //file with ExcelApplication.
            xla = new ExcelApplication();
            wb = xla.Open(Page.MapPath(@"templates\template.xlsx"));

            //Get the string list of the selectedCountries using the utility
            //method described below.
            List<string> selectedCountries = GetListBoxSelections();

            //Iterate through the selected countries in the string list
            //to make copies of the "SimpleTemplate" in the original
            //template.xlsx file.
            //Each new worksheet is placed after the "SimpleTemplate"
            //sheet and is named based on the selected country.
            for (int i = 0; i < selectedCountries.Count; i++)
            {
                wb.Worksheets.CopySheet(wb.Worksheets[0], wb.Worksheets.Count, selectedCountries[i]);
            }
            
            //Hide the "SimpleTemplate" sheet so that only the sheets that are assigned
            //to a country are viewable.
            wb.Worksheets[0].Visibility = Worksheet.SheetVisibility.Hidden;
            
            //Select the first country sheet after the "SimpleTemplate" to be viewed
            //when the report is opened.
            wb.Worksheets[1].Select();
        }

        
        protected void BindCountryData(string selection)
        //This function binds the data that will be displayed on each country's page.
        {
            //Create a new DataBindProperties object for the template and set 
            //the DataBindingProperty to bind data based on the worksheet name.
            DataBindingProperties dbp = xlt.CreateDataBindingProperties();
            dbp.WorksheetName = selection;

            //Create a string array for the "Header" data markers and bind the row data.
            
            //For each worksheet, FiscalYear : FY 2008, TradeDivision: Foreign Trade Division,
            //and Country: the parameter supplied country name
            string[] specificInfo = { "FY 2008", "Foreign Trade Division", selection };
            
            //The header markers are "FiscalYear", "TradeDivision", and "Country"
            string[] headerTitles = { "FiscalYear", "TradeDivision", "Country" };
            
            //Bind the header data row to the data markers in each worksheet (%%=Header.FiscalYear etc) 
            xlt.BindRowData(specificInfo, headerTitles, "Header", dbp);

            //Get the data for the top 5 sales and all the sales within the year.
            //The data uses generic parsing from CSV files to access the appropriate tables.
            //Note that the data can come from anywhere. For more information,
            //see http://wiki.softartisans.com/display/EW8/ExcelTemplate.BindData.
            DataTable dts = GetCSVData(Page.MapPath("//data//"+selection+"5.csv"));
            DataTable dts2 = GetCSVData(Page.MapPath("//data//"+selection+"All.csv"));
            xlt.BindData(dts, "Top", dbp);
            xlt.BindData(dts2, "Details", dbp);

            //The ExtraDataMarkers were removed so the GetCSVData knew to not raise an error.
            xlt.RemoveExtraDataMarkers = true;
        }


        #region Utility Methods
        /// <summary>
        /// Gets the items that were selected in the list box.
        /// </summary>
        /// <parameter>The string of filename</parameter>
        /// <returns>A populated datatable</returns>
        DataTable GetCSVData(string csvFileName)
        {
            DataTable dt;
            using (GenericParserAdapter parser = new GenericParserAdapter(csvFileName))
            {
                parser.ColumnDelimiter = ',';
                parser.FirstRowHasHeader = true;

                dt = parser.GetDataTable();
            }
            return dt;
        }

        /// <summary>
        /// Gets the items that were selected in the list box.
        /// </summary>
        /// <returns>A string list of the selected items</returns>
        protected List<string> GetListBoxSelections(){
        //Get the ListBox selections and make the appropriate number of copies
            List<string> countryNames = new List<string>();
            foreach (int i in ListBox1.GetSelectedIndices())
            {
                countryNames.Add(ListBox1.Items[i].Text);
            }
            return countryNames;
        }
        #endregion
    }
}