﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Data.SqlClient;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using GenericParsing;

/*Add this line to use ExcelWriter*/
using SoftArtisans.OfficeWriter.ExcelWriter;

namespace ExtendedSalesSummary
{
    public partial class Part2_CSV : System.Web.UI.Page
    {
        /*Declare the global variables that will be used throughout the code.
        * This includes the ExcelApplication object, the ExcelTemplate object,
        * and the Workbook object.*/
        private ExcelApplication xla;
        private ExcelTemplate xlt;
        private Workbook wb;

        protected void Page_Load(object sender, EventArgs e)
        {
        }

        protected void RunReport_Click(object sender, EventArgs e)
        //This function is run when the "Run Report" Button is clicked.
        {
            try
            {
                //Generate the report
                GenerateReport();
            }
            catch (Exception ex)
            {
                ErrorLabel.Visible = true;
                if (ex.Message.Contains("General license key exception"))
                {
                    ErrorLabel.Text = "OfficeWriter does not appear to be" +
                       " installed on this machine. If you have not " +
                       "purchased OfficeWriter, you can download the " +
                       "evaluation version here: " +
                       "http://www.officewriter.com/officewriter-free-evaluation.aspx";
                }
                else
                {
                    ErrorLabel.Text = ex.Message;
                }

            }
        }

        protected void GenerateReport()
        {
            //Format the template to deal with many different countries
            GenerateTemplate();

            //Add a cover sheet formatted using the ExcelWriter API
            AddCoverSheet();

            //Populate the template by binding the data
            PopulateTemplate();

            //Process and stream the final report
            xlt.Process();
            xlt.Save(Page.Response, "output.xlsx", false);
        }

        protected void PopulateTemplate()
        {
            //Create a new ExcelTemplate object and open the existing 
            //ExcelApplication.
            xlt = new ExcelTemplate();
            xlt.Open(xla, wb);

            //Iterate through the worksheets using a for loop 
            //For each worksheet in the workbook, bind the country data
            //using the BindCountryData. This function requires the name of
            //the worksheet as a string parameter.
            for (int i = 2; i < wb.Worksheets.Count; i++)
            {
                BindCountryData(wb.Worksheets[i].Name);
            }
            
            //Populate the cover sheet with the data defined in the function.
            BindCoverData();
        }

        protected void AddCoverSheet()
        {
            //Create a new Worksheet called "Summary" at the beginning of the workbook.
            //This sheet will hold the company logo, date created, report for, 
            //and table of contents fields.
            wb.Worksheets.CreateWorksheet("Summary", 0);

            //Give the "Summary" sheet the Worksheet variable 'ws'
            Worksheet ws = wb.Worksheets["Summary"];

            //Hide the gridlines for the "Summary" sheet
            ws.ShowGridlines = false;

            //Create the global style describing underlined cells
            GlobalStyle underlined = wb.CreateStyle();
            underlined.Font.Underline = Font.UnderlineStyle.Single;
            
            //Create the global style describing bold, size 12 font cells
            GlobalStyle boldAndFont12 = wb.CreateStyle();
            boldAndFont12.Font.Bold = true;
            boldAndFont12.Font.Size = 12;
            
            //Make the value of cell A15 "Date Executed :" and apply the underlined style
            ws.Cells["A15"].Value = "Date Executed :";
            ws.Cells["A15"].Style = underlined;

            //Make the value of cell A16 "Report For :" and apply the underlined style
            ws.Cells["A16"].Value = "Report For :";
            ws.Cells["A16"].Style = underlined;

            //Make the value of cell A18 "Table of Contents" and apply the bold, size 12 font style
            ws.Cells["A18"].Value = "Table of Contents";
            ws.Cells["A18"].Style = boldAndFont12;

            //Insert the data markers for the WebFormData (Textbox inputted name, and today's date) as strings.
            //Data markers must have the format %%=[DataSource].[ColumnName]
            ws.Cells["C16"].Value = "%%=WebFormData.Name";
            ws.Cells["C15"].Value = "%%=WebFormData.Date";

            //Iterate through the country sheet names, excluding the "Summary" and hidden "SimpleTemplate" sheets.
            //Cells below the "Table of Contents" cell are given a Hyperlink formula to another sheet in the workbook.
            //When hyperlinking to another page in a workbook, the formula must follow
            //"=HYPERLINK(\"#[DestinationSheet]![DesitinationCell]\", \"[Text]\")"
            for (int i = 2; i < wb.Worksheets.Count; i++)
            {
                string sheetName = wb.Worksheets[i].Name.ToString();
                ws.Cells[16 + i, 1].Formula = "=HYPERLINK(\"#"+sheetName+"!A1\", \""+sheetName+"\")";
            }

            //Create an anchor for the picture.
            Anchor anc = wb.Worksheets[0].CreateAnchor(0, 0, 0, 0);
            
            //Get the selected radio button value, which was set to the path of the image.
            //Note that this is a utility method and not an OfficeWriter capability.
            string image = RBImage.SelectedItem.Value;

            //Create the picture object and set the path and anchor
            Picture logo = ws.Pictures.CreatePicture(Page.MapPath(@image), anc);

            //Set the dimensions of the picture
            logo.Height = 195;
            logo.Width = 290;

            //Select the pages that you want to see when you first open the document
            wb.Worksheets[0].Select();
        }

        
        protected void BindCoverData()
        {
            //Get the name that was typed into the text box and save as string variable "name"
            string name = TextBox1.Text;

            //Create a string array for the "Header" data markers and bind the row data.

            //For each worksheet, Name : textbox input, and Date: today's date 
            //and Country: the parameter supplied country name
            string[] coverData = {name, DateTime.Now.Date.ToString("M/dd/yyyy")};

            //The WebFormData data markers are "Name" and "Date"
            string[] coverMarkers = { "Name", "Date" };

            //Bind the WebFormData data row to the data markers on the cover sheet (%%=WebFormData.Name etc)
            //with blank DataBindingProperties.
            xlt.BindRowData(coverData, coverMarkers, "WebFormData", xlt.CreateDataBindingProperties());

            //The ExtraDataMarkers were removed so the GetCSVData knew to not raise an error
            xlt.RemoveExtraDataMarkers = true;
        }
        

        public void GenerateTemplate()
        {
            //Create a new ExcelApplication and open the existing template 
            //file with ExcelApplication.
            xla = new ExcelApplication();
            wb = xla.Open(Page.MapPath(@"templates\template.xlsx"));

            //Get the string list of the selectedCountries using the utility
            //method described below.
            List<string> selectedCountries = GetListBoxSelections();

            //Iterate through the selected countries in the string list to make copies of the "SimpleTemplate" 
            //in the original template.xlsx file. Each new worksheet is placed after the "SimpleTemplate"
            //sheet and is named based on the selected country.
            for (int i = 0; i < selectedCountries.Count; i++)
            {
                wb.Worksheets.CopySheet(wb.Worksheets[0], wb.Worksheets.Count, selectedCountries[i]);
            }

            //Hide the "SimpleTemplate" sheet so that only the sheets that are assigned
            //to a country are viewable.
            wb.Worksheets[0].Visibility = Worksheet.SheetVisibility.Hidden;  
        }

        
        protected void BindCountryData(string selection)
        //This function binds the data that will be displayed on each country's page.
        {
            //Create a new DataBindProperties object for the template and set 
            //the DataBindingProperty to bind based on the worksheet name.
            DataBindingProperties dbp = xlt.CreateDataBindingProperties();
            dbp.WorksheetName = selection;

            //Create a string array for the "Header" data markers and bind the row data.

            //For each worksheet, FiscalYear : FY 2008, TradeDivision: Foreign Trade Division,
            //and Country: the parameter supplied country name
            string[] specificInfo = { "FY 2008", "Foreign Trade Division", selection };

            //The header markers are "FiscalYear", "TradeDivision", and "Country"
            string[] headerTitles = { "FiscalYear", "TradeDivision", "Country" };

            //Bind the header data row to the data markers in each worksheet (%%=Header.FiscalYear etc) 
            xlt.BindRowData(specificInfo, headerTitles, "Header", dbp);

            //Get the data for the top 5 sales and all the sales within the year.
            //The data uses generic parsing from CSV files to access the appropriate tables.
            //Note that the data can come from anywhere. For more information,
            //see http://wiki.softartisans.com/display/EW8/ExcelTemplate.BindData.
            DataTable dts = GetCSVData(Page.MapPath("//data//" + selection + "5.csv"));
            DataTable dts2 = GetCSVData(Page.MapPath("//data//" + selection + "All.csv"));
            xlt.BindData(dts, "Top", dbp);
            xlt.BindData(dts2, "Details", dbp);
        }

        #region Utility methods
        /// <summary>
        /// Gets the items that were selected in the list box.
        /// </summary>
        /// <parameter>The string of filename</parameter>
        /// <returns>A populated datatable</returns>
        DataTable GetCSVData(string csvFileName)
        {
            DataTable dt;
            using (GenericParserAdapter parser = new GenericParserAdapter(csvFileName))
            {
                parser.ColumnDelimiter = ',';
                parser.FirstRowHasHeader = true;

                dt = parser.GetDataTable();
            }
            return dt;
        }

        /// <summary>
        /// Gets the items that were selected in the list box.
        /// </summary>
        /// <returns>A string list of the selected items</returns>
        protected List<string> GetListBoxSelections()
        {
            //Get the ListBox selections and make the appropriate number of copies
            List<string> countryNames = new List<string>();
            foreach (int i in ListBox1.GetSelectedIndices())
            {
                countryNames.Add(ListBox1.Items[i].Text);
            }
            return countryNames;
        }
        #endregion
    }
}